;;; -*- Lisp -*-
;;;
;;; examples of various random things
;;;
;;;   add a stem-down
;;;   whole rests
;;;   the text x and y fields
;;;   drum beats
;;;   staccato above the beam
;;;   extensions for all-stems-up, or all-eighths, etc
;;;   silly beams
;;;   alternate rhythm names
;;;   slurs going off into space (invisible notes)
;;;   pizzicato entity
;;;   gray-scale example
;;;   how to pass parameters to a user-defined graphics entity
;;;   how to apply some attribute to everything on a staff
;;;   arrows between objects (uses filter-hook)
;;;   page numbers (uses page-hook) and line separation marks (uses line-hook)
;;;   lining up onsets across staves and setting the onset of a tied-to staff automatically
;;;   fun with PostScript level 2 patterns (example of prolog)
;;;   silly note-heads
;;;   partial staves
;;;   glissando across a clef change
;;;   justification abbreviations
;;;   changing margins without affecting justification decisions
;;;   multi-note tremolos
;;;   doubled unison
;;;   crescendo hairpin with text inside
;;;   clusters
;;;   an example of beaming across multiple percussion clefs
;;;   frames (a simple example)



;;; ------------------------------------------------
;;; add a stem down and make the note head a half-note head (carl.cmn)
;;; stem up would be the same thing, but placed at (+ (x0 note) .2875) and going up .75, not down .75

(setf with-stem-down-too 
    (make-instance 'self-acting
	:action #'(lambda (note &rest rest)
		    (declare (ignore rest))
		    (setf (head-quarters note) 2) ;half-note note head value
		    (add-to-marks 
		     note 
		     (list (make-instance 'sundry
			       :name :extra-stem-down
			       :mark #'(lambda (mark note score &rest rest)
					 (declare (ignore rest))
					 (moveto score (x0 note) (y0 note))
					 (setf (line-width score) (stem-width score))
					 (rlineto score 0 -.75) ;second stem going down
					 (setf (line-width score) 0))))))
	:arguments nil))



;;; ------------------------------------------------
;;; send out NUM whole-rests (can be scaled by 0 0 if invisible rests are needed for an attached staff) (joh.cmn)

(defun whole-rests (num)
  (engorge (loop for i from 1 to num collect whole-rest collect bar)))



;;; ------------------------------------------------
;;; an example of using the X and Y fields in the text class (joh.cmn)

(defun staff-text- (&rest args)
  (apply #'text- unjustified (connecting-pattern '(7 20))
	 (font-name "Times-Italic") (font-scaler .4)
	 (y #'(lambda (mark note score &rest rest) 
		(+ (staff-y0 note) 1.0 (dy mark))))
	 (x #'(lambda (mark note score &rest rest) 
		(+ (x0 note) (if (sign note) .2 0))))
	 args))



;;; ------------------------------------------------
;;; use auxiliary-note to create a drum beat in small notes

(defvar ax-note-size .8)

(defun ax (&rest args) 
  (apply #'auxiliary-note no-beam (scale ax-note-size ax-note-size) args))

(defun axes (&rest args) 
  (let* ((xdif .5) 
	 (xtotal (+ .25 (* xdif (length args)))))
    (engorge 
     (loop for arg in args and i from xtotal downto 0 by xdif 
      collect (ax arg e (dx (- i)))))))

(cmn staff treble (e5 q (axes d4 d4 d4 d4)))



;;; ------------------------------------------------
;;; staccato placed above the beam (as in mir.cmn)

(defun display-beam-staccato (mark note score &rest rest)
  (declare (ignore rest))
  (let* ((y0 (+ (%staff-y0 note) .25 (y1 note))))
    (moveto score 
	    (+ (box-x0 note) (box-x0 mark) (dxy-dx mark) (center note) -.05)
	    (+ y0 (box-y0 mark) (dxy-dy mark)))
    (simple-show score %dot)))

(defvar beam-staccato (make-instance 'write-protected-sundry :name :beam-staccato :mark #'display-beam-staccato))
(defun beam-staccato (&rest objects) (apply #'mark #'display-beam-staccato :beam-staccato objects))



;;; ------------------------------------------------
;;; with-stem-up

(defun with-stem-up (&rest args) 
  (engorge 
   (loop for arg in args 
    collect (if (or (note-p arg) 
		    (chord-p arg)) 
		(progn 
		  (setf (stem-direction arg) :up) 
		  arg) 
	      arg))))

;;; (cmn staff treble c5 q (with-stem-up (c5 q) (c4 q)) c5 q)


;;; or similarly:

(defun with-eighths (&rest args) 
  (engorge (loop for arg in args collect (if (note-p arg) (apply #'note arg e nil) arg))))

;;; (cmn staff treble (with-eighths c4 e4 f4 g4))



;;; ------------------------------------------------
;;; silly beams
;;;
;;; (this explicit beaming stuff can be useful if cmn screws up)

(cmn staff treble (c4 e. (begin-beam (explicit-beams (list (list 1 1 0) (list 0 1 0) (list 0 0 0))))) 
     (c4 s) (c4 s end-beam))

;;; that is, the various beam makers take the additional message explicit-beams.
;;; the argument to explicit-beams is a list of three lists, each list having a
;;; number for each note under the beam, and each number referring to (respectively)
;;;
;;;  the number of full beams going to the right from the current note
;;;  the number of partial beams heading right from the current note
;;;  the number of partial beams heading left from the current note
;;;
;;; so ((1 1 0) (0 1 0) (0 0 0)) => 1 full beam across the top, and one partial beam
;;; heading toward the right from the second note.

(cmn staff treble (c4 e. (begin-beam (dy 1) (explicit-beams (list (list 1 1 0) (list 5 1 0) (list 0 0 3))))) 
     (c4 s) (c4 s end-beam))


;;; ------------------------------------------------
;;; other rhythm names
;;;
(defvar 8th (rq 1/2))
(defvar 16th (rq 1/4))
(defvar 32nd (rq 1/8))
(defvar 64th (rq 1/16))
(defvar 128th (rq 1/32))


;;; ------------------------------------------------
;;; slurs going off into emptiness (ringing sounds and so on)
;;;
(cmn staff treble (g5 w begin-slur) (a5 w (scale 0 0) end-slur) bass (cs2 w begin-slur (scale 0 0)) (cs2 w end-slur))



;;; ------------------------------------------------
;;; pizzicato (other such names could be handled similarly)
;;;
(defun display-pizzicato (mark note score &rest rest)
  (declare (ignore rest))
  (display (text "pizz." unjustified (font-name "Times-Italic") (font-scaler .5) 
		 (x0 (- (+ (box-x0 note) (dxy-dx note) (dxy-dx mark)) .25))
		 (y0 (- (+ (%staff-y0 note) (dxy-dy mark)) .5)))
	   nil score rest))

(defvar pizzicato (make-instance 'write-protected-sundry :name :pizzicato :mark #'display-pizzicato))
(defun pizzicato (&rest objects) (apply #'mark #'display-pizzicato :pizzicato objects))



;;; ------------------------------------------------
;;; fun with gray-scale
;;;
(cmn (free-expansion-factor 2.0) staff (treble (gray-scale .8)) 
  (g5 (note-head :square) w (gray-scale .7) (begin-slur (gray-scale .6))) (g5 (scale 0 0) end-slur w) 
  (g5 (note-head :square) w (gray-scale .5) (begin-slur (gray-scale .4))) (g5 (scale 0 0) end-slur w) 
  (g5 (note-head :square) w (gray-scale .3) (begin-slur (gray-scale .2))) (g5 (scale 0 0) end-slur w) 
  (g5 (note-head :square) w (gray-scale .1) (begin-slur)) (g5 (scale 0 0) end-slur w) 
  bass (c2 (scale 0 0) (begin-tie (dashed '(.2 .1)))) (cs2 w end-tie))



;;; ------------------------------------------------
;;; an example of passing parameters to a user-supplied mark
;;;
;;; (cmn staff treble c4 q wind-block) 
;;; (cmn staff treble (c4 q (wind-block 4 1)))

(defun display-wind-block-1 (width height mark note score &rest rest)
    (let* ((y-off (staff-y0 note))
           (x-off (x0 note)))
      (setf (line-width score) .025)
      (moveto score x-off y-off)
      (lineto score (+ x-off 0) (+ y-off height))
      (lineto score (+ x-off width) (+ y-off height))
      (lineto score (+ x-off width) (+ y-off 0))
      (lineto score (+ x-off 0) (+ y-off 0))
      (fill-in score)
      (setf (line-width score) 0)))

(defun display-wind-block (mark note score &rest rest)
  (apply #'display-wind-block-1 5 1 mark note score rest))

(defvar wind-block (make-instance 'write-protected-sundry :name ':wind-block :mark #'display-wind-block))

(defun wind-block (width height &rest objects)
   (apply #'mark #'(lambda (mark note score &rest rest)
		     (apply #'display-wind-block-1 width height mark note score rest))
	  :wind-block objects))


;;; ------------------------------------------------
;;; examples of applying some attribute to every note on a staff
;;;
;;; (cmn staff always-stem-down treble c4 q)

(defvar always-stem-down 
    (make-self-acting  
     :action #'(lambda (staff &rest rest) 
		 (loop for obj in (staff-data staff) do 
		   (if (or (note-p obj) (chord-p obj)) 
		       (setf (stem-direction obj) :down)))) 
     :arguments nil))

(setf always-tie-up
    (make-self-acting 
     :action #'(lambda (staff &rest rest)
		 (loop for obj in (staff-data staff) do 
		   (if (or (note-p obj) (rest-p obj) (chord-p obj)) 
		       (setf (tie-direction obj) :up)))) 
     :arguments nil))

(setf always-slur-up
    (make-self-acting 
     :action #'(lambda (staff &rest rest)
		 (loop for obj in (staff-data staff) do 
		   (if (or (note-p obj) (chord-p obj)) 
		       (setf (slur-direction obj) :up)))) 
     :arguments nil))




;;; ------------------------------------------------
;;; arrows pointing to the current object from some other saved object
;;;
;;; uses filter-hook called in filterify, just for lafs.
;;; can set arrow color, dashed-style, arrowhead length and angle,
;;;   and the distances we fall back from the two objects.


(defvar filter-hook-progn nil)

(defun arrow-dashed (arg) (list :arrow-dashed arg))
(defun arrow-begin-space (arg) (list :arrow-begin-space arg))
(defun arrow-end-space (arg) (list :arrow-end-space arg))
(defun arrowhead-length (arg) (list :arrowhead-length arg))
(defun arrowhead-angle (arg) (list :arrowhead-angle arg))
(defun arrow-color (arg) (list :arrow-color arg))

(defun arrow-from (start-obj &rest args)
  ;; obj is the object where the arrow starts
  ;; args can be arrow-dashed, arrow-begin-space, arrow-end-space, arrow-color, arrowhead-length, arrowhead-angle
  ;; we'll handle these as simple functions, rather than define a new arrowhead class and use self-acting
  (if (not filter-hook)
      (setf filter-hook #'(lambda (score)
			    (let ((saved-progn filter-hook-progn))
			      (setf filter-hook-progn nil)
			      (loop for fun in saved-progn do
				(funcall fun score))
			      score))))
  (make-self-acting 
   :action 
   #'(lambda (end-obj &rest rest)
       (push 
	#'(lambda (score) 
	    (add-to-marks		;this extra indirection needed to make sure we end up on the right page
	     end-obj 
	     (list 
	      (make-instance 'sundry 
		  :name :arrow-from 
		  :mark #'(lambda (mark note score &rest rest)
			    (let* ((ox0 (+ (x0 start-obj) (center start-obj) (dxy-dx start-obj)))
				   (oy0 (y0 start-obj))
				   (ox1 (+ (x0 end-obj) (center end-obj) (dxy-dx end-obj)))
				   (oy1 (y0 end-obj))
				   (theta (if (= oy1 oy0) 0.0
					    (if (= ox1 ox0) 
						(* pi -.5) 
					      (atan (- oy1 oy0) (- ox1 ox0)))))
				   (arrow-dashed nil)
				   (arrow-fatness (/ pi 8))
				   (arrow-length .4)
				   (arrow-color 0)
				   (arrow-end-space .5)
				   (arrow-begin-space .25))
			      (loop for arg in args do
				(case (first arg)
				  (:arrow-dashed (setf arrow-dashed (second arg)))
				  (:arrow-begin-space (setf arrow-begin-space (second arg)))
				  (:arrow-end-space (setf arrow-end-space (second arg)))
				  (:arrowhead-length (setf arrow-length (second arg)))
				  (:arrowhead-angle (setf arrow-fatness (second arg)))
				  (:arrow-color (setf arrow-color (second arg)))))
			      (let* ((top-theta (+ theta arrow-fatness))
				     (bottom-theta (- theta arrow-fatness))
				     (x0 (coerce (+ ox0 (* arrow-begin-space (cos theta))) 'single-float))
				     (x1 (coerce (- ox1 (* arrow-end-space (cos theta))) 'single-float))
				     (top-x (coerce (- x1 (* arrow-length (cos top-theta))) 'single-float))
				     (bottom-x (coerce (- x1 (* arrow-length (cos bottom-theta))) 'single-float))
				     (y0 (coerce (+ oy0 (* arrow-begin-space (sin theta))) 'single-float))
				     (y1 (coerce (- oy1 (* arrow-end-space (sin theta))) 'single-float))
				     (top-y (coerce (- y1 (* arrow-length (sin top-theta))) 'single-float))
				     (bottom-y (coerce (- y1 (* arrow-length (sin bottom-theta))) 'single-float))
				     (back-x (coerce (- x1 (* .5 arrow-length (cos theta))) 'single-float))
				     (back-y (coerce (- y1 (* .5 arrow-length (sin theta))) 'single-float)))
				(set-gray-scale score arrow-color)
				(moveto score x0 y0)
				(lineto score x1 y1)
				(draw score :pattern (if arrow-dashed (if (not (listp arrow-dashed)) (list 5 10) arrow-dashed)))
				(moveto score x1 y1)
				(lineto score top-x top-y)
				(lineto score back-x back-y)
				(lineto score bottom-x bottom-y)
				(lineto score x1 y1)
				(fill-in score)
				(set-gray-scale score 0)
				)))))))
	filter-hook-progn))
   :arguments nil))

;;; (cmn (staff treble c4 q (setf bigc (c4 w)) (c4 q) line-break d4 e d4 e d4 q (arrow-from bigc (arrow-dashed '(10 16)))))
;;; (cmn (staff treble c4 q (setf bigc (c4 w)) (c4 q) line-break d4 e d4 e d4 q 
;;;        (arrow-from bigc (arrowhead-angle (/ pi 4))(arrowhead-length 1.0) (arrow-color .6) (arrow-dashed '(10 13)))))
;;; (cmn (staff treble c4 q (setf bigc (c4 w)) (c5 q stem-up) b4 e a4 e g4 q 
;;;       (arrow-from bigc (arrow-dashed '(2 4)) (arrowhead-length .3) (arrow-begin-space .4) 
;;;       (arrow-end-space .25) (arrowhead-angle (/ pi 12)))))




;;; ------------------------------------------------
;;; Page numbers 
;;;
;;; There's a variable called page-hook that is funcall'd
;;; on each page.  It's intended for page-numbers, borders,
;;; and so on.  Here's an example:

(defun page-number (score page)
  ;; called automatically if placed as value of page-hook
  ;; dimensions of current page are in score variables page-width, page-height,
  ;;   left/right/header/footer-margin (in inches), "page" above is the current page number
  (moveto score 
	  (in-inches (+ (right-margin score) (* .5 (page-width score)) -.5))
	  (in-inches (* .75 (footer-margin score))))	;bottom center
  (show score (text (format nil "-~D-" page) (font-name "Times-Roman") (font-scaler .5))))


;;; To use this, 
;;;
;;; (setf page-hook #'page-number)
;;;
;;; then call up cmn as usual.
;;;
;;; (setf page-hook nil) 
;;;
;;; to cancel it.



;;; ------------------------------------------------
;;; line marks
;;;
;;; Similar to page-hook is the variable line-hook -- it's value (a function) is funcall'd
;;; on each line.  The arguments passed to that function are:
;;;
;;;   (score top-staff current-line-number y1 x0 x1 data-x0 staff-size)
;;;
;;; where the actual musical data goes from data-x0 to x1, starting vertically at y1.
;;; Current-line-number is the number of this line within the current page (starting at 1).
;;; 
;;; Here, we'll cause each line after the first on the page to add the // marks that are
;;; sometimes used to emphasize the separation between successive lines of a multi-system score.

(defun separate-lines (score staff line sy1 sx0 sx1 sdx0 ssize)
  (when (> line 1)			;i.e. no need for these marks at the top of the page
    (add-to-marks 
     staff 
     (list (make-instance 'sundry 
	     :name :line-separation
	     :mark #'(lambda (mark note score &rest rest)
		       (let* ((fx / 1.0 ssize)
			      (fatness (* fx .125))
			      (length (* fx .5))
			      (slant (* fx .25))
			      (ysep (+ fatness (* fx .075)))
			      (x0 (* fx sdx0))
			      (y1 (* fx sy1))
			      (y0 (+ y1 (* fx .5 (line-separation score)))))
			 (slanted-fill-in score x0 y0 (+ x0 length) (+ y0 slant) fatness)
			 (slanted-fill-in score x0 (- y0 ysep) (+ x0 length) (+ y0 slant (- ysep)) fatness))))))))



;;; ------------------------------------------------
;;; lining up notes across staves
;;;
;;; it can be a pain to figure out what a note's onset should be
;;;   and add in all the necessary rests and so forth.  Here we
;;;   use CLOS's :after method combination to line two notes up
;;;   automatically.

(defvar n1 nil)
(defvar n2 nil)

;;; these are our note tags -- n1's onset will be passed along to n2

(defmethod (setf odb-onset) :after (val (hi note-mixin)) 
  (if (eq hi n1) (setf (onset n2) val)))

;;; and an example:

(cmn (staff treble g4 h (setf n1 (c4 e)) d4 e e4 w) 
     (staff bass d3 e (setf n2 (c3 e)) a2 w))

;;; and using a tied-to staff:

(cmn (setf s1 (staff treble g4 h (setf n1 (c4 e stem-down)) (d4 e stem-down) e4 w)) 
     (staff (tied-to s1) treble (setf n2 (c5 q stem-up)) a5 w))


;;; you can also use the (eql form) parameter specializer:

(setf n1 (c4 e))
(defmethod (setf odb-onset) :after (val (hi (eql n1))) (setf (onset n2) val))

(cmn (staff treble g4 h n1 d4 e e4 w) (staff bass d3 e (setf n2 (c3 e)) a2 w))



;;; ------------------------------------------------
;;; use PostScript level 2 patterns -- here we draw our usual
;;;   staff, treble clef, etc using stars.
;;; 
;;; taken largely from /NextDeveloper/Examples/PostScript/Level2Stars.eps
;;; This only works on a Level 2 system (i.e. NeXT 3.0)

(cmn (size 200) (prolog #'(lambda (score)
		 (let ((prog "
    % Star code mostly from chapter 4 of the Second Edition
    % of the PostScript Language Reference Manual
    
    8 dict dup begin
	/PatternType    1 def
	/PaintType      1 def
	/TilingType     1 def
	/BBox           [0 0 40 40] def
	/XStep          40 def
	/YStep          40 def
	/star		{
	    gsave 0 8 moveto 4 {144 rotate 0 8 lineto} repeat
	    closepath fill grestore
	} def
	/PaintProc      {
	    begin
	    0.333 setgray 0 0 40 40 rectfill
	    1.0 0.0 0.0 setrgbcolor 10 10 translate star
	    0.0 1.0 0.0 setrgbcolor 0 20 translate star
	    0.0 0.0 1.0 setrgbcolor 20 -20 translate star
	    1.0 1.0 0.0 setrgbcolor 0 20 translate star
	    end
	} def
    end
    matrix makepattern
    /starpattern exch def

    /setStarPattern {
        starpattern setpattern
    } def     
"))
		   (if (c-output-in-progress score)
		       (c-print prog)
		     (format (output score) "~A" prog)))))

   (staff (pattern #'(lambda (score obj)
		       (values "setStarPattern"
			       ""
			       "")))
	  treble c4 q))



;;; ------------------------------------------------
;;; silly note-heads
;;;
;;; the note head can be any mark or glyph, as the following shows:

(cmn staff treble 
  (c5 e (note-head %pedal)) (d5 e (note-head (glyph %trebleclef (scale .5 .5)))) 
  (e5 e (note-head %accent)) (f5 e (note-head %fermata)) 
  (g5 e (note-head (smiley-face (dy -.25)))) 
  (c6 e (note-head (smiley-face (dy -.2)))))

;;; (we're using the smiley-face code from the cmn documentation).



;;; ------------------------------------------------
;;; one way to do partial staves

(cmn (staff treble g4 e e4 e c4 q) 
  (setf s1 (staff (x1 1.8) bass (c3 q (scale 0 0)))) 
  (staff (tied-to s1) (x0 3) (c3 (onset 1) q)))



;;; ------------------------------------------------
;;; begin/end-glissando and glissando-to
;;;
;;; If there's a clef change between the begin-glissando and the end-glissando,
;;;   cmn still tries to match up the actual notes.  There are cases where this
;;;   looks strange, but these cases can use glissando-to:

(cmn staff treble (c4 q begin-glissando) bass (g3 q end-glissando))
(cmn staff treble (c4 q (glissando-to g3)) bass (g3 q))



;;; ------------------------------------------------
;;; justification abbreviations
;;;
(defvar below (justification :below))
(defvar above (justification :above))
(defvar up (justification :up))
(defvar down (justification :down))



;;; ------------------------------------------------
;;;   changing margins without affecting justification decisions
;;;

(setf new-margin 2.0)

(cmn (size (round (* 24 (/ (- 8.5 (* 2 new-margin)) 7.5))))
     ;;              ^ old size    ^ fix-up for changed margin width
     (left-margin new-margin)
     (right-margin new-margin)
     ...)


;;; ------------------------------------------------
;;; multi-note tremolos
;;;

(cmn staff treble (c4 s (note-head :tremolo)) (d4 s (note-head :tremolo)) (e4 s (note-head :tremolo)))


;;; ------------------------------------------------
;;; double-unison

(cmn (chord (notes g4 g4) q))


;;; ------------------------------------------------
;;;   crescendo hairpin with text inside

(cmn (free-expansion-factor 3) (size 48) 
  (g4 h (crescendo (width .5) (end-dynamic (text "poco" (dx -.75) (dy .05) (font-scaler .3) (font-name "Times-Italic"))))) 
  g4 h bar)


;;; ------------------------------------------------
;;; clusters
;;;
;;; There are a zillion ways to draw clusters.  Here's one:


(defun display-cluster (mark chord score &rest rest)
  ;; can't be implemented as a stem-mark because the cluster might not have a stem.
  (when (and (or (not (member :just-fooling rest))
		 (not (eq (visible-justification chord) :none)))
	     (not (invisible-matrix-p chord)))
    (let* ((x0 (+ (x0 chord) (dxy-dx mark) (center chord)))
	   (low-line (minimum-line chord))
	   (high-line (maximum-line chord))
	   (low-y0 (+ (staff-y0 chord) (* (staff-line-separation score) low-line)))
	   (high-y0 (+ (staff-y0 chord) (* (staff-line-separation score) high-line)))
	   (filled (< (quarters chord) 2))
	   (stemmed (< (quarters chord) 4))
	   (stemright (and stemmed (member (stem-direction chord) '(:up :up?)))))
      (if filled
	  (fill-in score :rectangle t :closepath t 
		   :path (list (+ x0 (if stemright .15 -.15))  
			       low-y0 
			       (if stemright -.25 .25)
			       (- high-y0 low-y0)))
	(progn
	  (setf (line-width score) .04)
	  (moveto score (+ x0 .125) (+ low-y0 .05))
	  (rlineto score 0 (- high-y0 low-y0 .15))
	  (rmoveto score -.25 0)
	  (rlineto score 0 (- (- high-y0 low-y0 .15)))
	  (simple-draw score))))))

(defvar cluster (make-instance 'write-protected-sundry :name :cluster :mark #'display-cluster))
(defun cluster (&rest objects) (apply #'mark #'display-cluster :cluster objects))

;;; (cmn (size 100) (chord (notes c4 c5) q (cluster)) (chord (notes c4 c5) w (cluster)) (chord (notes g4 g5) q (cluster)))



;;; ------------------------------------------------
;;;   an example of beaming across multiple percussion clefs
;;;

(cmn (automatic-rests nil) (staff-separation 0)
  (staff (staff-lines 1) (start-line 2) percussion 
    (b4 s (setf hi (beam- (dy 2)))) )
  (staff (staff-lines 1) (start-line 2) percussion 
    (b4 s (onset .25) (-beam- hi)) (b4 s (-beam- hi) (onset .75)))
  (staff (staff-lines 1) (start-line 2) percussion 
    (b4 s (onset .5) (-beam hi)) (sixteenth-rest (scale 0 0)) bar))



;;; ------------------------------------------------
;;; frames
;;;
;;; this coule be extended to add arrows and so on.


(defclass frame (tag-mixin sundry)
  ((width :initform .025 :initarg :width :accessor width)))

(defun frame- (&rest args)
  (let ((nf (make-instance 'frame)))
    (loop for arg in args do
      (if (self-acting-p arg)
	  (funcall (action arg) nf (arguments arg))))
    (make-self-acting 
     :action #'(lambda (note newf) 
		 (setf (tag-note newf) note) 
		 nil)
     :arguments nf)))

(defun -frame (oldf &rest args)
  (let ((newf (make-instance 'frame)))
    (loop for arg in args do
      (if (self-acting-p arg)
	  (funcall (action arg) newf (arguments arg))))
    (make-self-acting
     :action #'(lambda (note old-frame)
		 (add-to-marks note
			       (list (make-sundry
				      :name :frame
				      :mark #'(lambda (mark lnote score &rest rest)
						(when (not (member :just-fooling rest))
						  (let* ((first-note (tag-note old-frame))
							 (x0 (+ (min (box-x0 first-note) (box-x0 lnote)) (dxy-dx old-frame) -.1))
							 (x1 (+ (max (box-x0 first-note) (box-x0 lnote)) (dxy-dx newf) .4))
							 (y0 (+ (min (box-y0 first-note) (box-y0 lnote)) (dxy-dy newf) -.2))
							 (y1 (+ (max (box-y0 first-note) (box-y0 lnote)) (dxy-dy old-frame) .4)))
						    (setf (line-width score) (width old-frame))
						    (moveto score x0 y0)
						    (rlineto score 0 (- y1 y0))
						    (rlineto score (- x1 x0) 0)
						    (rlineto score 0 (- y0 y1))
						    (rlineto score (- x0 x1) 0)
						    (simple-draw score)
						    (setf (line-width score) 0))))))))
     :arguments (arguments oldf))))

;;; (cmn staff treble (c4 q (setf fr (frame- (width .1) (dy .5)))) d4 q e4 q (f4 q (-frame fr (dx .1) (dy -.25))))
